import json
from typing import List

import requests
from bs4 import BeautifulSoup

from .c_exceptions import NoPicturesFound
from .dtypes import (
    PictureSize, ZeroChanCategory,
    ZeroChanImage, ZeroChanPage, SortBy
)


class ZeroChan:
    WEBSITE_URL = "https://www.zerochan.net"

    # TODO: filter by image size by pixel, alt and other

    def __init__(self):
        self._search = ""
        self._page = 1
        self._sort_by = SortBy.LAST
        self._size = PictureSize.ALL_SIZES
        self.cookies = dict(
            z_lang="en"
        )
        self.req_args = {}

    def size(self, size: PictureSize) -> 'ZeroChan':
        self._size = size
        return self

    def sort(self, sort_by: SortBy) -> 'ZeroChan':
        self._sort_by = sort_by
        return self

    def page(self, page_num: int) -> 'ZeroChan':
        self._page = page_num
        return self

    def search(self, title: str) -> 'ZeroChan':
        self._search = title
        return self

    def authorize(self, z_hash: str, z_id: str):
        self.cookies.update(dict(
            z_hash=z_hash,
            z_id=z_id
        ))
        return self

    def _get_soup(self) -> BeautifulSoup:
        self.req_args.update(dict(
            p=self._page,
            s=self._sort_by,
            d=int(self._size)
        ))
        req = requests.get(
            f"{self.WEBSITE_URL}/{self._search}",
            params=self.req_args,
            cookies=self.cookies
        )
        return BeautifulSoup(req.content, "html.parser")

    def category(self) -> ZeroChanCategory:
        soup = self._get_soup()
        content_el = soup.find("script", {"type": "application/ld+json"})
        if content_el is None:
            raise NoPicturesFound
        page_data = json.loads("".join(content_el.contents))
        menu = soup.find("div", dict(id="menu"))
        # parsing description
        p_tags = menu.find_all("p")
        description = p_tags[1].text.replace("\r\n", "")
        return ZeroChanCategory(
            name=page_data.get("name"),
            image=page_data.get("image"),
            type=page_data.get("@type"),
            description=description
        )

    def _parse_pics(self, pics_soup: BeautifulSoup) -> List[ZeroChanImage]:
        images = []
        for pic in pics_soup.find_all("li"):
            if getattr(pic.a, "img") is None:
                # parser try to parse "Members only" link if not authorized
                # idk why it's going do
                continue
            title = pic.a.img.get("title")
            splitted_title = title.split(" ")
            size = int(splitted_title[1][:-2])  # remove "kb" chars
            height, width = map(int, list(splitted_title[0].split("x")))  # split by x and convert to int

            pic_download_el = pic.p.a
            if pic_download_el.img:
                download_url = pic.p.a.get("href")
            else:
                download_el = pic_download_el.parent.find_all("a")[1]
                download_url = download_el.get("href")
            images.append(
                ZeroChanImage(title=title, url=download_url, height=height, width=width, kbsize=size)
            )
        return images

    def pics(self) -> ZeroChanPage:
        soup = self._get_soup()
        pics_soup = soup.find("ul", dict(id="thumbs2"))

        if pics_soup is None:
            raise NoPicturesFound
        imgs = self._parse_pics(pics_soup)

        # Setting page and maxpage to 1 if random mode chosen
        if self._sort_by == SortBy.RANDOM:
            page = 1
            max_page = 1
        else:
            paginator_el = soup.find("p", {"class": "pagination"})
            str_list = paginator_el.text.strip().replace("\t", " ").split(" ")
            if self._page == 1:
                page = int(str_list[1])
                max_page = int(str_list[3])
            else:
                page = int(str_list[4])
                max_page = int(str_list[6])
        return ZeroChanPage(
            images=imgs,
            page=page,
            max_page=max_page
        )



from dataclasses import dataclass
from enum import IntEnum, Enum
from typing import List


@dataclass
class ZeroChanCategory:
    name: str
    image: str
    type: str
    description: str


@dataclass
class ZeroChanImage:
    title: str
    url: str
    height: int  # in px
    width: int  # in px
    kbsize: int  # in KB

    @property
    def size(self):
        return f"{self.height}x{self.width}"

    # TODO: Download method?


@dataclass
class ZeroChanPage:
    images: List[ZeroChanImage]
    page: int
    max_page: int


class PictureSize(IntEnum):
    ALL_SIZES = 0
    BIGGER_AND_BETTER = 1
    BIG_AND_HUGE = 2


class SortBy(str, Enum):
    LAST = "id"
    POPULAR = "fav"
    RANDOM = "random"



import argparse
import datetime
import os

import requests

from .c_exceptions import NoPicturesFound
from .dtypes import SortBy, PictureSize
from .lib import ZeroChan

PICS_DIR = os.path.join(os.getcwd(), "zerochan_pics")

pic_sizes = {
    1: PictureSize.ALL_SIZES,
    2: PictureSize.BIGGER_AND_BETTER,
    3: PictureSize.BIG_AND_HUGE
}

sorted_by = {
    1: SortBy.LAST,
    2: SortBy.POPULAR,
    3: SortBy.RANDOM
}


def super_duper_logger(text: str, level: str):
    """This is my secret development, shh!"""
    print(f"[{datetime.datetime.now()}] [{level}]: {text}")


def create_parser():
    # TODO: change prog
    parser = argparse.ArgumentParser(
        epilog="March 2021"
    )

    parser.add_argument(
        "--title",
        help="Title name to parse",
        required=True
    )
    parser.add_argument(
        "-s",
        "--size",
        default=1,
        type=int,
        choices=[1, 2, 3],
        help="Image size: 1 - All (default), 2 - bigger and better, 3 - large only"
    )
    parser.add_argument(
        "-m",
        "--mode",
        default=1,
        type=int,
        choices=[1, 2, 3],
        help="Sort mode: 1 - Last (default), 2 - Popular, 3 - Random"
    )
    parser.add_argument(
        "-p",
        "--page",
        default=1,
        type=int,
        help="Start page to download"
    ),
    parser.add_argument(
        "-a",
        "--authorize",
        metavar="z_hash;z_id",
        help="z_hash and z_id from cookies like z_hash;z_id"
    )
    parser.add_argument(
        "-c",
        "--check_mode",
        action="store_true",
        help="Only check mode, don't download pics, just print urls."
    )
    parser.add_argument(
        "-o",
        "--onlypage",
        help="Download only selected page",
        action="store_true"
    )
    return parser


def download_file(url: str, path: str):
    filename = url.split('/')[-1]
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(os.path.join(path, filename), 'wb') as f:
            for chunk in r.iter_content(chunk_size=2048):
                f.write(chunk)
    super_duper_logger(
        f"Downloaded {filename} to {path}", "DOWNLOADER"
    )


def get_pics(zerochan_instance: ZeroChan):
    try:
        return zerochan_instance.pics()
    except NoPicturesFound:
        super_duper_logger("No pictures found on page.", "ERROR")


if __name__ == "__main__":
    parser_instance = create_parser()
    args = parser_instance.parse_args()

    zerochan = ZeroChan()\
        .search(args.title)\
        .size(pic_sizes.get(args.size))\
        .sort(sorted_by.get(args.mode))\
        .page(args.page)

    if args.authorize:
        z_hash, z_id = args.authorize.split(";")
        zerochan.authorize(z_hash, z_id)

    data = get_pics(zerochan)

    page = data.page
    if args.onlypage:
        max_page = page
    else:
        max_page = data.max_page

    if not args.check_mode:
        call_dir = os.getcwd()
        try:
            os.mkdir(PICS_DIR)
        except FileExistsError:
            pass

    for i in range(page, max_page):
        super_duper_logger(f"Get page {i}/{max_page}", "REQUEST")
        data_page = get_pics(zerochan.page(i))
        if data_page is None:
            continue
        for image in data_page.images:
            super_duper_logger(
                f"Getting image {image.title}: {image.url} with size {image.size} {image.kbsize}",
                "PARSER"
            )
            if not args.check_mode:
                download_file(image.url, PICS_DIR)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zerochan']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'zerochan',
    'version': '0.3.0',
    'description': 'Python library and download cli util for https://www.zerochan.net',
    'long_description': '# Zerochan\n\nLibrary for Zerochan.net with pics parsing and downloader included!\n\n## Features\n* CLI utility for pics downloading from zerochan.net\n* Library for create custom downloader (you can write own) or data analyze.\n* Strong typed!\n\n## Installation:\n\n### Using pip\n`pip install zerochan`\n### Using poetry\n`poetry add zerochan`\n\n## Using as library: \n\nFirst, you should create `Zerochan` instance:\n```python\nfrom zerochan import ZeroChan\n\nzerochan_instance = ZeroChan()\n```\n\nNow, you can set some args for request\n\n```python\nfrom zerochan import ZeroChan, PictureSize, SortBy\n\nzerochan = ZeroChan()\n\nzerochan.search("Spain")  # Set title to search\nzerochan.size(PictureSize.BIGGER_AND_BETTER) # Set quality and pic size\nzerochan.sort(SortBy.POPULAR) # Set sorting (now only popular)\nzerochan.page(1) # Page to parse\nzerochan.authorize("hjsaf7afkjsaf78", "127364") # Authorize by z_hash and z_id in cookies\n```\n\n...or set args like this:\n\n```python\nzerochan.search("Spain")\\\n    .size(PictureSize.BIGGER_AND_BETTER)\\\n    .sort(SortBy.POPULAR)\n```\n\nAfter all settings, you should call `.pics()` to get pics:\n\n```python\ndata = zerochan.pics()\nfor img in data.images:\n    print(img.url)\n```',
    'author': 'kiriharu',
    'author_email': 'kiriharu@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kiriharu/zerochan',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
